@@ -42,6 +42,24 @@ class BroadpeakIoApi(BpkioSdkConsumer):
         session_file: Optional[str] = None,
         **kwargs,
     ):
+        """
+        Initialize the Broadpeak.io API client.
+
+        You can specify either a tenant label or an API key.
+        If you specify a tenant label, the API key will be retrieved from the `~/.bpkio/tenant` file.
+
+        If you don't specify either, the SDK will use environment variables `BPKIO_TENANT` or `BPKIO_API_KEY`.
+        If those are not set, the SDK will check whether there is a default tenant configured in the `~/.bpkio/tenant` file.
+
+        See the [Authentication](../guides/authentication.md) guide for more information.
+
+        Args:
+            tenant: The label of the Tenant Profile to use (from the `~/.bpkio/tenant` file).
+            api_key: The tenant's API key.
+            fqdn: The FQDN of the Broadpeak.io entrypoint to use.
+            use_cache: Whether to use the cache.
+            session_file: The file to record the session to.
+        """
         self.use_cache = use_cache
         if tenant and api_key:
             raise ValueError("You can't specify both tenant and api_key")
@@ -78,8 +96,10 @@ class BroadpeakIoApi(BpkioSdkConsumer):
             self._fqdn = t.fqdn
 
         else:
-            raise InvalidTenantError("You must specify either api_key or tenant, "
-                                      "or have configured a default tenant")            
+            raise InvalidTenantError(
+                "You must specify either api_key or tenant, "
+                "or have configured a default tenant"
+            )
 
         base_url = f"https://{self._fqdn}/v1/"
 
@@ -138,11 +158,12 @@ class BroadpeakIoApi(BpkioSdkConsumer):
             raise InvalidApiKeyFormat(reason=e)
 
     def parse_api_key(self) -> dict:
-        """Parses the API Key (token) and extract the information it contains"""
+        """Parses the API Key and extracts the information it contains"""
         return self._parse_api_key(self._api_key)
 
     @staticmethod
     def is_valid_api_key_format(string: str):
+        """Checks if the API Key is in the format expected by broadpeak.io"""
         try:
             BroadpeakIoApi._parse_api_key(string)
             return True
@@ -150,11 +171,12 @@ class BroadpeakIoApi(BpkioSdkConsumer):
             return False
 
     def get_tenant_id(self) -> int:
+        """Returns the tenant ID from the API Key"""
         return self.parse_api_key().get("tenantId")
 
     @SessionRecorder.do_not_record
     def get_self_tenant(self) -> Tenant:
-        """Get tenant linked to the current API Key"""
+        """Returns the Tenant resource linked to the current API Key"""
         tenant_id = self.get_tenant_id()
 
         # Yet another workaround because /tenants/me does not work for Tenant 1
@@ -178,9 +200,11 @@ class BroadpeakIoApi(BpkioSdkConsumer):
 
     @property
     def fqdn(self):
+        """Get the FQDN of the Broadpeak.io entrypoint"""
         return self._fqdn
 
     def uses_default_fqdn(self):
+        """Check if the FQDN is the default one"""
         return self._fqdn == DEFAULT_FQDN
 
     @staticmethod
@@ -197,8 +221,16 @@ class BroadpeakIoApi(BpkioSdkConsumer):
 
     @staticmethod
     def is_correct_entrypoint(url: str, api_key: str, verify_ssl) -> bool | str:
-        """Checks that the URL is a valid Broadpeak.io entrypoint"""
+        """Checks whether a URL is a valid Broadpeak.io entrypoint
 
+        Args:
+            url: The URL to check.
+            api_key: The API key to use.
+            verify_ssl: Whether to verify the SSL certificate.
+
+        Returns:
+            bool: True if the URL is a valid Broadpeak.io entrypoint, False otherwise.
+        """
         try:
             api = BroadpeakIoApi(api_key=api_key, fqdn=url, verify_ssl=verify_ssl)
             api.get_self_tenant()
@@ -217,7 +249,7 @@ class BroadpeakIoApi(BpkioSdkConsumer):
         except BroadpeakIoApiError:
             raise BroadpeakIoApiError(
                 500,
-                "Unable to access the API. Check that your API key is valid",
+                f"Unable to access the API for tenant {tenant_id} (on {self.fqdn}). Check that your API key is valid",
                 None,
                 None,
             )
