@@ -1,5 +1,3 @@
-from datetime import datetime
-
 from uplink import Query, get, response_handler, returns
 
 from bpkio_api.consumer import BpkioSdkConsumer
@@ -18,6 +16,6 @@ class ConsumptionApi(BpkioSdkConsumer):
         self,
         start_time: Query("start-time"),  # type: ignore
         end_time: Query("end-time"),  # type: ignore
-        tenant: Query("tenantId"),
+        tenant: Query("tenantId"),  # type: ignore
     ) -> ConsumptionData:  # type: ignore
         """Get the consumption data between 2 dates"""
