@@ -17,7 +17,11 @@ from uplink import (
 from bpkio_api.caching import cache_api_results
 from bpkio_api.consumer import BpkioSdkConsumer
 from bpkio_api.exceptions import BroadpeakIoHelperError, ResourceExistsError
-from bpkio_api.helpers.list import collect_from_ids, get_all_with_pagination
+from bpkio_api.helpers.list import (
+    collect_from_ids,
+    collect_from_sparse_resources,
+    get_all_with_pagination,
+)
 from bpkio_api.helpers.objects import find_duplicates_of
 from bpkio_api.helpers.search import SearchMethod, search_array_with_filters
 from bpkio_api.helpers.upsert import UpsertOperationType, upsert_status
@@ -82,9 +86,23 @@ class ServicesApi(BpkioSdkConsumer):
 
     # === Helpers ===
 
-    @cache_api_results("list_services")
-    def list(self):
-        return get_all_with_pagination(self._get_page, count_fn=self.count)
+    @cache_api_results("list_services", key_suffix_from_arg="sparse")
+    def list(self, sparse: bool = True):
+        """List all services
+
+        Args:
+            sparse (bool, optional): Whether to return sparse services, or resolve them into fully qualified resources. Defaults to True. Note that resolving services will incur a performance penalty.
+
+        Returns:
+            List[Svc.ServiceSparse | Svc.AdInsertionService | Svc.ContentReplacementService | Svc.VirtualChannelService | Svc.AdaptiveStreamingCdnService]: List of services
+        """
+        sparse_services = get_all_with_pagination(self._get_page, count_fn=self.count)
+        if sparse:
+            return sparse_services
+        else:
+            return collect_from_sparse_resources(
+                sparse_resources=sparse_services, get_fn=self._get_by_type
+            )
 
     def search_by_type(self, type: Svc.ServiceType) -> List[Svc.ServiceSparse]:
         all_items = self.list()
@@ -124,6 +142,20 @@ class ServicesApi(BpkioSdkConsumer):
         services = self.list()
         return search_array_with_filters(services, filters=filters)
 
+    def _get_by_type(
+        self,
+        service_id: int,
+        type: Svc.ServiceType,
+    ):
+        if type == Svc.ServiceType.AD_INSERTION:
+            return self.ad_insertion.retrieve(service_id)
+        if type == Svc.ServiceType.VIRTUAL_CHANNEL:
+            return self.virtual_channel.retrieve(service_id)
+        if type == Svc.ServiceType.CONTENT_REPLACEMENT:
+            return self.content_replacement.retrieve(service_id)
+        if type == Svc.ServiceType.ADAPTIVE_STREAMING_CDN:
+            return self.adaptive_streaming_cdn.retrieve(service_id)
+
     def retrieve(
         self, service_id: int
     ) -> (
@@ -147,19 +179,16 @@ class ServicesApi(BpkioSdkConsumer):
             A specific sub-type of service
         """
 
+        # Find the sparse service with that id to find its type
         candidates = self.search(
             int(service_id), field="id", method=SearchMethod.STRICT
         )
         try:
             service = candidates[0]
-            if service.type == Svc.ServiceType.AD_INSERTION:
-                return self.ad_insertion.retrieve(service_id)
-            if service.type == Svc.ServiceType.VIRTUAL_CHANNEL:
-                return self.virtual_channel.retrieve(service_id)
-            if service.type == Svc.ServiceType.CONTENT_REPLACEMENT:
-                return self.content_replacement.retrieve(service_id)
-            if service.type == Svc.ServiceType.ADAPTIVE_STREAMING_CDN:
-                return self.adaptive_streaming_cdn.retrieve(service_id)
+            return self._get_by_type(
+                service_id,
+                service.type,
+            )
         except IndexError as e:
             raise BroadpeakIoHelperError(
                 message=f"There is no service with ID {service_id}",
@@ -243,11 +272,11 @@ class ServicesApi(BpkioSdkConsumer):
         Examples:
             ```python
             from bpkio_api.helpers.upsert import UpsertOperationType, upsert_status
-            
+
             # Create or update
             service = ContentReplacementServiceIn(name="News", description="News content")
-            result = api.upsert(service, if_exists="update")            
-            
+            result = api.upsert(service, if_exists="update")
+
             # Optional: Check the status of the operation
             status = upsert_status.get()
             if status == UpsertOperationType.CREATED:
