@@ -1,17 +1,22 @@
 from typing import Any, List, Tuple
 from urllib.parse import urlparse, urlunparse
 
+from uplink import Body, Query, delete, get, json, post, put, response_handler, returns
+
 from bpkio_api.caching import cache_api_results
 from bpkio_api.consumer import BpkioSdkConsumer
 from bpkio_api.exceptions import BroadpeakIoHelperError, ResourceExistsError
-from bpkio_api.helpers.list import collect_from_ids, get_all_with_pagination
+from bpkio_api.helpers.list import (
+    collect_from_ids,
+    collect_from_sparse_resources,
+    get_all_with_pagination,
+)
 from bpkio_api.helpers.objects import find_duplicates_of
 from bpkio_api.helpers.search import SearchMethod, search_array_with_filters
 from bpkio_api.models import Sources as S
-from bpkio_api.response_handler import postprocess_response
-from uplink import Body, Query, delete, get, json, post, put, response_handler, returns
+from bpkio_api.response_handler import extract_count, postprocess_response
 
-from .enums import UpsertOperationType
+from bpkio_api.helpers.upsert import UpsertOperationType, upsert_status
 
 
 @response_handler(postprocess_response)
@@ -49,9 +54,14 @@ class SourcesApi(BpkioSdkConsumer):
 
     @returns.json()
     @get("sources")
-    def get_page(self, offset: Query = 0, limit: Query = 5) -> List[S.SourceSparse]:  # type: ignore
+    def _get_page(self, offset: Query = 0, limit: Query = 5) -> List[S.SourceSparse]:  # type: ignore
         """Get a partial list of Sources"""
 
+    @response_handler(extract_count)
+    @get("sources")
+    def count(self) -> int:
+        """Get a count of all sources"""
+
     @returns.json()
     @post("sources/{type}/check")
     def check(
@@ -61,9 +71,47 @@ class SourcesApi(BpkioSdkConsumer):
 
     # === Helpers ===
 
-    @cache_api_results("list_sources")
-    def list(self):
-        return get_all_with_pagination(self.get_page)
+    @cache_api_results("list_sources", key_suffix_from_arg="sparse")
+    def list(self, sparse: bool = True):
+        """List all sources
+
+        Args:
+            sparse (bool, optional): Whether to return sparse sources, or resolve them into fully qualified resources. Defaults to True. Note that resolving sources will incur a performance penalty.
+
+        Returns:
+            List[S.SourceSparse | S.AssetSource | S.LiveSource | S.AdServerSource | S.AssetCatalogSource | S.SlateSource | S.OriginSource]: List of sources
+        """
+        sparse_sources = get_all_with_pagination(self._get_page, count_fn=self.count)
+        if sparse:
+            return sparse_sources
+        else:
+            return collect_from_sparse_resources(
+                sparse_resources=sparse_sources, get_fn=self._get_by_type
+            )
+
+    def _get_by_type(
+        self,
+        source_id: int,
+        source_type: S.SourceType,
+    ) -> S.SourceSparse:
+        match source_type:
+            case S.SourceType.ASSET:
+                return self.asset.retrieve(source_id)
+            case S.SourceType.ASSET_CATALOG:
+                return self.asset_catalog.retrieve(source_id)
+            case S.SourceType.LIVE:
+                return self.live.retrieve(source_id)
+            case S.SourceType.SLATE:
+                return self.slate.retrieve(source_id)
+            case S.SourceType.AD_SERVER:
+                return self.ad_server.retrieve(source_id)
+            case S.SourceType.ORIGIN:
+                return self.origin.retrieve(source_id)
+            case _:
+                raise BroadpeakIoHelperError(
+                    message=f"Source type not supported: {source_type}",
+                    original_message="",
+                )
 
     def search_by_type(self, type: S.SourceType) -> List[S.SourceSparse]:
         all_items = self.list()
@@ -131,24 +179,7 @@ class SourcesApi(BpkioSdkConsumer):
         candidates = self.search(int(source_id), field="id", method=SearchMethod.STRICT)
         try:
             source = candidates[0]
-            match source.type:
-                case S.SourceType.ASSET:
-                    return self.asset.retrieve(source_id)
-                case S.SourceType.ASSET_CATALOG:
-                    return self.asset_catalog.retrieve(source_id)
-                case S.SourceType.LIVE:
-                    return self.live.retrieve(source_id)
-                case S.SourceType.SLATE:
-                    return self.slate.retrieve(source_id)
-                case S.SourceType.AD_SERVER:
-                    return self.ad_server.retrieve(source_id)
-                case S.SourceType.ORIGIN:
-                    return self.origin.retrieve(source_id)
-                case _:
-                    raise BroadpeakIoHelperError(
-                        message=f"Source type not supported: {source.type}",
-                        original_message="",
-                    )
+            return self._get_by_type(source_id, source.type)
         except IndexError as e:
             raise BroadpeakIoHelperError(
                 message=f"There is no source with ID {source_id}",
@@ -196,37 +227,59 @@ class SourcesApi(BpkioSdkConsumer):
         source: S.SourceIn,
         if_exists: str = "retrieve",
         unique_fields: List[str | Tuple] = [],
-    ) -> Tuple[
+    ) -> (
         S.AssetSource
         | S.AdServerSource
         | S.AssetCatalogSource
         | S.SlateSource
         | S.LiveSource
-        | S.SourceIn,
-        UpsertOperationType,
-    ]:
-        """Creates a source with adaptable behaviour if it already exist.
+        | S.SourceIn
+    ):
+        """Create, retrieve, or update a source based on existence.
+
+        This method provides flexible handling of source creation with different
+        behaviors when the source already exists.
 
         Args:
-            source (SourceIn): The payload for the source to create
-            if_exists (str): What action to take if it exists:
-              `error` (default) returns an error;
-              `retrieve` returns the existing object;
-              `update` updates the existing object.
-            unique_fields (List[str | Tuple], optional): List of the fields
-            or combination of fields to check for unicity. Defaults to [].
+            source: The source data to create or update.
+            if_exists: Controls behavior when a source with the same name exists:
+
+                - `error`: Raise an error (default)
+                - `retrieve`: Return the existing source
+                - `update`: Update the existing source with new data
 
         Returns:
-            Tuple[AssetSource | AdServerSource | AssetCatalogSource | SlateSource | LiveSource | None, int]:
-            The resource created or retrieved, with an indicator: 1 = created, 0 = retrieved, 2 = updated, -1 = failed
+            The Source object (created, retrieved, or updated)
 
+        Raises:
+            ResourceExistsError: If the source exists and if_exists="error".
+            ValidationError: If the source data is invalid.
+
+        Examples:
+            ```python
+            from bpkio_api.helpers.upsert import UpsertOperationType, upsert_status
+
+            # Create or update
+            source = LiveSourceIn(name="News", description="News content")
+            result = api.upsert(source, if_exists="update")
+
+            # Optional: Check the status of the operation
+            status = upsert_status.get()
+            if status == UpsertOperationType.CREATED:
+                print("Created new source")
+            elif status == UpsertOperationType.UPDATED:
+                print("Updated existing source")
+            ```
         """
 
         try:
-            return (self.create(source), UpsertOperationType.CREATED)
+            created_source = self.create(source)
+            upsert_status.set(UpsertOperationType.CREATED)
+            return created_source
         except ResourceExistsError as e:
             if if_exists == "error":
-                return (source, UpsertOperationType.ERROR)
+                upsert_status.set(UpsertOperationType.ERROR)
+                return source
 
             unique_fields = list(set(unique_fields + ["name"]))
             for fld in unique_fields:
@@ -242,10 +295,12 @@ class SourcesApi(BpkioSdkConsumer):
                     existing_resource = self.retrieve(dupes[0][1].id)
 
                     if if_exists == "retrieve":
-                        return (existing_resource, UpsertOperationType.RETRIEVED)
+                        upsert_status.set(UpsertOperationType.RETRIEVED)
+                        return existing_resource
                     elif if_exists == "update":
                         updated_resource = self._update(existing_resource.id, source)
-                        return (updated_resource, UpsertOperationType.UPDATED)
+                        upsert_status.set(UpsertOperationType.UPDATED)
+                        return updated_resource
 
     def check_by_id(self, source_id: int) -> List[S.SourceStatusCheckResult]:
         source = self.retrieve(source_id=source_id)
@@ -297,7 +352,9 @@ class AssetSourcesApi(BpkioSdkConsumer):
     @json
     @returns.json(S.AssetSource)
     @put("sources/asset/{source_id}")
-    def update(self, source_id: int, source: Body(type=S.AssetSourceIn)) -> S.AssetSource:  # type: ignore
+    def update(
+        self, source_id: int, source: Body(type=S.AssetSourceIn)
+    ) -> S.AssetSource:  # type: ignore
         """Update an asset source"""
 
     @delete("sources/asset/{source_id}")
@@ -423,7 +480,9 @@ class AssetCatalogSourcesApi(BpkioSdkConsumer):
     @json
     @returns.json(S.AssetCatalogSource)
     @put("sources/asset-catalog/{source_id}")
-    def update(self, source_id: int, source: Body(type=S.AssetCatalogSourceIn)) -> S.AssetCatalogSource:  # type: ignore
+    def update(
+        self, source_id: int, source: Body(type=S.AssetCatalogSourceIn)
+    ) -> S.AssetCatalogSource:  # type: ignore
         """Updates an asset source"""
 
     @delete("sources/asset-catalog/{source_id}")
@@ -486,7 +545,9 @@ class AdServerSourcesApi(BpkioSdkConsumer):
     @json
     @returns.json(S.AdServerSource)
     @put("sources/ad-server/{source_id}")
-    def update(self, source_id: int, source: Body(type=S.AdServerSourceIn)) -> S.AdServerSource:  # type: ignore
+    def update(
+        self, source_id: int, source: Body(type=S.AdServerSourceIn)
+    ) -> S.AdServerSource:  # type: ignore
         """Update an Ad Server source"""
 
     @delete("sources/ad-server/{source_id}")
@@ -587,7 +648,9 @@ class SlateSourcesApi(BpkioSdkConsumer):
     @json
     @returns.json(S.SlateSource)
     @put("sources/slate/{source_id}")
-    def update(self, source_id: int, source: Body(type=S.SlateSourceIn)) -> S.SlateSource:  # type: ignore
+    def update(
+        self, source_id: int, source: Body(type=S.SlateSourceIn)
+    ) -> S.SlateSource:  # type: ignore
         """Update a slate source"""
 
     @delete("sources/slate/{source_id}")
@@ -650,7 +713,9 @@ class OriginSourcesApi(BpkioSdkConsumer):
     @json
     @returns.json(S.OriginSource)
     @put("sources/origin/{source_id}")
-    def update(self, source_id: int, source: Body(type=S.OriginSourceIn)) -> S.OriginSource:  # type: ignore
+    def update(
+        self, source_id: int, source: Body(type=S.OriginSourceIn)
+    ) -> S.OriginSource:  # type: ignore
         """Update an origin source"""
 
     @delete("sources/origin/{source_id}")
